<?php
require 'Cache.php';

class CacheRouter {
  var $map = array();
  var $exp = array();
  var $settings = array();

  function __construct() {
    global $conf;
    $conf['page_cache_fastpath'] = TRUE;
    
    $this->statistic_enable = $conf['cacherouter_statistic'];
    
    $this->__construct_exp();
  }
  
  private function __construct_exp() {
    global $conf;
    
    foreach ($conf['cacherouter'] as $bin => $options) {
      $exp = explode('#', $bin);
      if (count($exp) > 1) {
        $this->exp[$exp[0]][$exp[1]] = $bin;
      }
    }
  }
  
  private function __exp_keyed_bin($key, $bin) {
    if (empty($this->exp)) return $bin;
    if (empty($this->exp[$bin])) return $bin;
    foreach ($this->exp[$bin] as $key_ptr => $real_bin) if (preg_match('/'.$key_ptr.'/', $key)) return $real_bin;
    return $bin;
  }
  
  private function __init($bin) {
    global $conf;
    
    if (isset($conf['cacherouter'][$bin]['engine']) && !isset($this->map[$bin])) {
      $type = strtolower($conf['cacherouter'][$bin]['engine']);
    }
    else {
      $type = isset($conf['cacherouter']['default']['engine']) ? $conf['cacherouter']['default']['engine'] : 'db';
    }
    if (!class_exists($type . 'Cache')) {
      if (!require(dirname(__FILE__) .'/engines/' . $type . '.php')) {
        return FALSE;
      }
    }
    $cache_engine = $type . 'Cache';
    
    $this->map[$bin] =& new $cache_engine($bin);
  }
  
  private function statistic($bin, $key, $op) {
    if (!$this->statistic_enable) return;
    
    db_query('INSERT INTO {cacherouter_statistic} (t,b,k,o) VALUES(%d,"%s","%s",%d)', time(), $bin, $key, $op);
  }
  
  public function get($key, $bin) {
    $bin = $this->__exp_keyed_bin($key, $bin);
    
    if (!isset($this->map[$bin])) {
      $this->__init($bin);
    }
    $this->statistic($bin, $key, 0);
    return $this->map[$bin]->get($key);
  }
  
  public function set($key, $value, $expire, $headers, $bin) {
    $bin = $this->__exp_keyed_bin($key, $bin);
    
    if (!isset($this->map[$bin])) {
      $this->__init($bin);
    }
    $this->statistic($bin, $key, 1);
    return $this->map[$bin]->set($key, $value, $expire, $headers);
  }
  
  public function delete($key, $bin) {
    $bin = $this->__exp_keyed_bin($key, $bin);
    
    if (!isset($this->map[$bin])) {
      $this->__init($bin);
    }
    $this->statistic($bin, $key, 2);
    return $this->map[$bin]->delete($key);
  }
  
  public function flush($bin) {
    if (!isset($this->map[$bin])) {
      $this->__init($bin);
    }
    
    if ($this->exp[$bin]) {
      foreach($this->exp[$bin] as $key_ptr => $real_bin) {
        if (!isset($this->map[$real_bin])) {
          $this->__init($real_bin);
        }
        $this->statistic($real_bin, '', 3);
        $this->map[$real_bin]->flush();
      }
    }
    $this->statistic($bin, '', 3);
    return $this->map[$bin]->flush();
  }
  
  public function page_fast_cache($bin) {
    if (!isset($this->map[$bin])) {
      $this->__init($bin);
    }
    
    if ($this->exp[$bin]) {
      foreach ($this->exp[$bin] as $key_ptr => $real_bin) {
        if (!isset($this->map[$real_bin])) {
          $this->__init($real_bin);
        }
        $this->map[$real_bin]->page_fast_cache();
      }
    }
    
    return $this->map[$bin]->page_fast_cache();
  }
}
