<?php
/** cacherouter.analize.inc --- 
 *
 * Copyright (C) 2009 Kayo Phoenix
 *
 * Author: Kayo Phoenix <kayo@controll.trk.tvs>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

if (!function_exists('html2rgb')):
  function html2rgb($color) {
    if ($color[0] == '#')
      $color = substr($color, 1);
    
    if (strlen($color) == 6)
      list($r, $g, $b) = array($color[0].$color[1],
                               $color[2].$color[3],
                               $color[4].$color[5]);
    elseif (strlen($color) == 3)
      list($r, $g, $b) = array($color[0].$color[0], $color[1].$color[1], $color[2].$color[2]);
    else
      return false;
    
    $r = hexdec($r); $g = hexdec($g); $b = hexdec($b);
    
    return array($r, $g, $b);
  }
endif;

function html2cssrgb($color) {
  $rgb = html2rgb($color);
  if (is_array($rgb)) return 'rgb('.implode(',', $rgb).')';
}

function rgb2css($rgb) {
  if (is_array($rgb)) return 'rgb('.implode(',', $rgb).')';
}

function rgb_interp($colors, $value) {
  $bk = 0.0;
  $ek = 1.0;
  foreach ($colors as $c) {
    $k = $c[0];
    $v = $c[1];
    if ($k == $value) return $v;
    if ($k <= $value && $k >= $bk) { $bk = $k; $bv = $v; }
    if ($k >= $value && $k <= $ek) { $ek = $k; $ev = $v; }
  }
  $d = ($value - $bk) / ($ek - $bk);
  $r = array();
  for ($i=0;$i<3;$i++) {
    $r[$i] = round(($be[$i] - $bv[$i]) * $d + $bv[$i]);
  }
  return $r;
}

function _cacherouter_chart($data) {
  $pth = drupal_get_path('module', 'cacherouter').'/jquery.svg/';
  
  drupal_add_js('misc/jquery.js', 'core');
  
  drupal_add_css($pth.'jquery.svg.css');
  drupal_add_js($pth.'jquery.svg.pack.js');
  drupal_add_js($pth.'jquery.svggraph.pack.js');
  
  $colors = array(array(0.0, array(169, 173, 188)), array(1.0, array(212, 212, 212)));
  
  {
    global $theme;
    theme();
    $palette = (object)variable_get('color_'.$theme.'_palette', NULL);
    if ($palette && $palette->top && $palette->bottom) {
      $colors = array(array(0.0, html2rgb($palette->top)), array(1.0, html2rgb($palette->bottom)));
    }
  }
  
  $num = count($data);
  $itr = 0;
  foreach ($data as $key => $value) {
    $js_data .= '.addSeries("'.$key.'", ['.$value.'], "'.rgb2css(rgb_interp($colors, $itr / $num)).'", "none")';
    $itr++;
  }
  
  drupal_add_js(
    '$(function(){'.
    '$("#chart").svg();'.
    'var svg=$("#chart").svg("get");'.
    'svg.graph.noDraw().title("")'.
    $js_data.
    '.type("pie"/*,{explode:[1]}*/)'.
    '.format("none","none")'.
    '.xAxis.title("").scale(0, 3).ticks(1, 0).labels([""]).end()'.
    '.yAxis.title("Percentage").scale(0, 10).ticks(10, 5).end()'.
    '.legend.settings({fill:"none",stroke:"none"}).show(true).area(0.6,0.1,0.9,0.9).end()'.
    '.area(0.0,0.1,0.5,0.9).redraw();'.
    '});',
    'inline'
  );
  
  return '<div id="chart" style="width: 700px; height: 400px;"></div>';
}

function _cacherouter_stat_usage($bin = NULL, $key = NULL) {
  $bin !== NULL && $cond[] = 'b="'.db_escape_string($bin).'"';
  $key !== NULL && $cond[] = 'k="'.db_escape_string($key).'"';
  $cond && $cond = ' WHERE '.implode(' AND ', $cond);
  
  for ($r = db_query('SELECT o AS op, COUNT(*) AS count FROM {cacherouter_statistic}'.$cond.' GROUP BY o ORDER BY o');
       $o = db_fetch_object($r); ) {
    switch ($o->op) {
    case 0: $get = $o->count; break;
    case 1: $set = $o->count; break;
    case 2: $del = $o->count; break;
    case 3: $fls = $o->count; break;
    }
  }
  
  $all = $get + $set + $del + $fls;
  
  $result->absolute = array(
    t('Number of queries') => $all,
    t('Number of get queries') => $get,
    t('Number of set queries') => $set,
    t('Number of delete queries') => $del,
    t('Number of flush queries') => $fls,
  );
  
  if ($all > 0) {
    $result->percentage = array(
      t('Percentage of get queries') => round($get / $all, 4),
      t('Percentage of set queries') => round($set / $all, 4),
      t('Percentage of delete queries') => round($del / $all, 4),
      t('Percentage of flush queries') => round($fls / $all, 4),
    );
  }
  
  if ($set > 0 && $get > 0) {
    $result->ratio = array(
      t('Ratio of get queries by set queries') => round($get / $set, 4),
      t('Ratio of set queries by get queries') => round($set / $get, 4),
    );
  }
  
  return $result;
}

function cacherouter_analize_usage() {
  $data = _cacherouter_stat_usage();
  
  $output .= _cacherouter_chart($data->percentage);
  $output .= theme('table', array_keys($data->absolute), array(array_values($data->absolute)));
  
  return $output;
}

function _cacherouter_stat_bins() {
  $cond = '';
  for ($r = db_query('SELECT b AS bin, COUNT(*) AS count FROM {cacherouter_statistic}'.$cond.' GROUP BY b ORDER BY COUNT(*) DESC');
       $o = db_fetch_object($r); $result[$o->bin] = $o->count);
  return $result;
}

function cacherouter_analize_bins() {
  $data = _cacherouter_stat_bins();
  $output .= _cacherouter_chart($data);  
  
  foreach ($data as $k => $v) $rows[] = array($k, $v);
  $output .= theme('table', array(t('Bin'), t('Queries')), $rows);
  
  return $output;
}

function _cacherouter_stat_keys() {
  $cond = '';
  $c = 20;
  for ($i = 0, $r = db_query('SELECT k AS k, COUNT(*) AS count FROM {cacherouter_statistic}'.$cond.' GROUP BY k ORDER BY COUNT(*) DESC');
       ($o = db_fetch_object($r)) && $i < $c; $result[$o->k] = $o->count, $i++);
  return $result;
}

function cacherouter_analize_keys() {
  $data = _cacherouter_stat_keys();
  $output .= _cacherouter_chart($data);
  
  foreach ($data as $k => $v) $rows[] = array($k, $v);
  $output .= theme('table', array(t('Key'), t('Queries')), $rows);
  
  return $output;
}
