<?php

  /*
   *  Method ``branger'':
   *  Case:
   *    Number of get requests much more than number of set requests.
   *  Optimality condition:
   *    frontend - more dynamic cache engine (usually more slow for get data and more fast for set data in comparsion with backend);
   *    backend - more preferred cache engine (usually fastest for get data and slowest for set data);
   *  Algorithm:
   *    In set case: Set data to frontend and delete data from backend.
   *    In get case: Try get from backend, if data exists return it. Otherwise try get from frontend and if this data existed more than "delay" seconds then set it into backend. Return this data if exists.
   *
   *  Method: ``frengor'':
   *  Case:
   *    ....
   *  Optimality condition:
   *    frontend - ....
   *
   */

  /* Кэшируемые данные хранятся в двух хранилищах, различающихся по скорости чтения/записи в них и общему объему. Фронтэнд хранилище имеет более быстрый двигатель кэша, но ограничено по объему хранимых данных. Ограничение по объему данных или числу хранимых значений не следует рассматривать буквально, будем считать, что мы не заинтересованы в слишком сильном разрастании объема кешируемых в этом хранилище данных. Бэкэнд хранилище не ограничено по объему, использует двигатель, более быстрый для операций чтения и медленный для записи. Помним, что наша основная цель - соблюсти баланс. Алгоритм работает следующим образом: при установки значений в кэш, реальная запись данных всегда осуществляется в фронтэнд хранилище с одновременным удалением устаревших данных в бэкэнде. Когда происходит запрос извлечения значения, пробуем сперва извлечь данные из бэкэнда, если их там нет - извлекаем из фронтэнда. В случае, если данные кэша в фронтэнд хранилище не пусты и просуществовали некоторое время, переносим их в бэкэнд, а в фронтэнде удаляем.
   * Также был реализован другой алгоритм. В отличии от предыдущего данные всегда запрашиваются сначала из фронтэнда, затем если их там нет - из бэкэнда и устанавливаются в фронтэнд. При записи данные пишутся в бэкэнд. Далее, чтобы редко запрашиваемые данные перемещались из фронтэнда в бэкэнд, надо что-то придумать. 
   */

class dualCache extends Cache {
  var $content = array();
  
  function page_fast_cache() {
    return $this->fast_cache;
  }
  
  function __construct($bin) {
    global $conf;
    
    // Configure backend router
    ($this->backend->bin = $conf['cacherouter'][$bin]['backend']) &&
      $conf['cacherouter'][$this->backend->bin] ||
      ($this->backend->bin = 'default');
    // Configure frontend router
    ($this->frontend->bin = $conf['cacherouter'][$bin]['frontend']) &&
      $conf['cacherouter'][$this->frontend->bin] ||
      ($this->frontend->bin = 'default');
    // Set controll method
    $this->method = $conf['cacherouter'][$bin]['method'];
    empty($this->method) && ($this->method = 'branger');
    // Set backend update delay
    $this->delay = $conf['cacherouter'][$bin]['delay'];
    $this->delay = is_numeric($this->delay) ? $this->delay : 60;
    // Check configuration
    $this->backend->bin != $this->frontend->bin &&
      ($this->backend->router = $this->__init($this->backend->bin)) &&
      ($this->frontend->router = $this->__init($this->frontend->bin)) ||
      ($this->incorrupt = TRUE) &&
      drupal_set_message(t('Dual cache engine for %s configured incorrectly.', array($bin)), 'error');
    
    parent::__construct($bin);
  }
  
  private function __init($bin) {
    global $conf;
    
    if (isset($conf['cacherouter'][$bin]['engine']) && !isset($this->map[$bin])) {
      $type = strtolower($conf['cacherouter'][$bin]['engine']);
    }
    else {
      $type = isset($conf['cacherouter']['default']['engine']) ? $conf['cacherouter']['default']['engine'] : 'db';
    }
    if (!class_exists($type . 'Cache')) {
      if (!require(dirname(__FILE__) . '/' . $type . '.php')) {
        return FALSE;
      }
    }
    $cache_engine = $type . 'Cache';
    
    return new $cache_engine($bin);
  }
  
  private function branger_get($key) {
    $cache = $this->backend->router->get($key);
    if ($cache) return $cache;
    
    $cache = $this->frontend->router->get($key);
    if ($cache && (time() - $cache->created > $this->delay)) {
      $this->backend->router->set($key, $cache->data, $cache->expire, $cache->headers);
      $this->frontend->router->delete($key);
    }
    
    return $cache;
  }
  
  private function branger_set($key, $value, $expire, $headers) {
    $this->frontend->router->set($key, $value, $expire, $headers);
    $this->backend->router->delete($key);
  }
  
  private function frengor_get($key) {
    $cache = $this->frontend->router->get($key);
    if ($cache) return $cache;
    
    $cache = $this->backend->router->get($key);
    if ($cache) // && (time() - $cache->created > $this->delay))
      $this->frontend->router->set($key, $cache->data, $cache->expire, $cache->headers);
    
    return $cache;
  }
  
  private function frengor_set($key, $value, $expire, $headers) {
    $this->frontend->router->set($key, $value, $expire, $headers);
    $this->backend->router->delete($key);
  }
  
  function get($key) {
    $function = $this->method . '_get';
    return $this->$function($key);
  }
  
  function set($key, $value, $expire = CACHE_PERMANENT, $headers = NULL) {
    $function = $this->method . '_set';
    return $this->$function($key, $value, $expire, $headers);
  }
  
  function delete($key) {
    $this->backend->router->delete($key);
    $this->frontend->router->delete($key);
  }
  
  function flush() {
    $this->backend->router->flush();
    $this->frontend->router->flush();
  }
}
